@@ -35,15 +35,17 @@ use warnings;
 
 use base 'Exporter';
 our @EXPORT = qw(
-		  allocate_colors
-		  allocate_color
-		  rgb_color
-		  rgb_color_opacity
-		  rgb_color_transparency
-		  rgb_to_color
-		  fetch_color
-aa_color
-	       );
+									allocate_colors
+									allocate_color
+									color_to_list
+									find_transparent
+									rgb_color
+									rgb_color_opacity
+									rgb_color_transparency
+									rgb_to_color
+									fetch_color
+									aa_color
+							 );
 
 use Carp qw( carp confess croak );
 use Digest::MD5 qw(md5_hex);
@@ -124,7 +126,7 @@ sub allocate_colors {
 	# now resolve name lookups
 	start_timer("colorlookups");
 	for my $color_name ( sort keys %{ $CONF{colors} } ) {
-		my $color_definition = $CONF{colors}{$color_name};
+		my $color_definition = lc $CONF{colors}{$color_name};
 		# if this color has already been allocated, skip it
 		next if exists $colors->{$color_name};
 		my %lookup_seen;
@@ -148,6 +150,9 @@ sub allocate_colors {
 	stop_timer("colortransparency");
 
 	# now resolve lists - employ caching since this can be slow (2-5 seconds);
+
+	goto SKIPLISTS if defined_and_zero(fetch_conf("color_lists_use"));
+
 	start_timer("colorlists");
 	my $hostname = hostname;
 	my $user     = $ENV{USERNAME} ? $ENV{USERNAME} . $PERIOD : $EMPTY_STRING;
@@ -176,38 +181,38 @@ sub allocate_colors {
 	my $is_cache_static = Circos::Configuration::fetch_configuration("color_cache_static");
 	my $rebuild_cache   = Circos::Configuration::fetch_configuration("color_cache_rebuild");
 	if ($rebuild_cache) {
-	    printdebug_group("cache","colorlist cache rebuild forced");
+		printdebug_group("cache","colorlist cache rebuild forced");
 	} elsif (-e $cache_file) {
-	    start_timer("colorcache");
-	    printdebug_group("cache","colorlist cache",$cache_file,"found");
-	    if ($is_cache_static || -M $cache_file < -M $CONF{configfile}) {
-		printdebug_group("cache","colorlist cache",$cache_file,"useable - static or more recent than configfile");
-		# cache file younger than config file, read cache
-		eval {
+		start_timer("colorcache");
+		printdebug_group("cache","colorlist cache",$cache_file,"found");
+		if ($is_cache_static || -M $cache_file < -M $CONF{configfile}) {
+			printdebug_group("cache","colorlist cache",$cache_file,"useable - static or more recent than configfile");
+			# cache file younger than config file, read cache
+			eval {
 		    $list_cache = retrieve($cache_file);
-		};
-		if ($@) {
+			};
+			if ($@) {
 		    printwarning("Problem reading color cache file $cache_file");
 		    $cache_ok = 0;
-		} else {
+			} else {
 		    printdebug_group("cache","colorlist cache",$cache_file,"read in");
 		    my $target_hash = Digest::MD5::md5_hex(join("", sort keys %{$CONF{colors}}));
 		    if ($list_cache->{colorhash} eq $target_hash) {
-			printdebug_group("cache","color list hash",$target_hash,"matches that of cache file - using cache file");
-			$cache_ok = 1;
+					printdebug_group("cache","color list hash",$target_hash,"matches that of cache file - using cache file");
+					$cache_ok = 1;
 		    } elsif ($is_cache_static) {
-			printdebug_group("cache","color list hash",$target_hash,"doesn't match that of cache file - using cache anyway because it is static");
-			$cache_ok = 1;
+					printdebug_group("cache","color list hash",$target_hash,"doesn't match that of cache file - using cache anyway because it is static");
+					$cache_ok = 1;
 		    } else {
-			printdebug_group("cache","color list hash",$target_hash,"does not match - colors changed? - recomputing file");
+					printdebug_group("cache","color list hash",$target_hash,"does not match - colors changed? - recomputing file");
 		    }
+			}
+		} else {
+			printdebug_group("cache","colorlist cache",$cache_file,"older than configfile - recreating cache");
 		}
-	    } else {
-		printdebug_group("cache","colorlist cache",$cache_file,"older than configfile - recreating cache");
-	    }
-	    stop_timer("colorcache");
+		stop_timer("colorcache");
 	} else {
-	    printdebug_group("cache","colorlist cache",$cache_file,"not found");
+		printdebug_group("cache","colorlist cache",$cache_file,"not found");
 	}
 	if (! $cache_ok) {
 		# create cache
@@ -264,6 +269,7 @@ sub allocate_colors {
 		push @$allocated_color_list, $color;
 	}
 	stop_timer("colorlists");
+ SKIPLISTS:
 	return $colors;
 }
  
@@ -375,8 +381,8 @@ sub validate_rgb_list {
 	return unless $r =~ /^\d+$/ && $r >= 0 && $r <= 255;
 	return unless $g =~ /^\d+$/ && $r >= 0 && $r <= 255;
 	return unless $b =~ /^\d+$/ && $r >= 0 && $r <= 255;
-	if(defined $a) {
-		if(is_number($a, "real", $strict, 0,1)) {
+	if (defined $a) {
+		if (is_number($a, "real", $strict, 0,1)) {
 			# ok
 		} else {
 			fatal_error("color","bad_alpha",$a);
@@ -397,8 +403,8 @@ sub validate_hsv_list {
 	return unless $h =~ /^\d+$/ && $h >= 0 && $h <= 360;
 	return unless is_number($s, "real", $strict, 0,  1);
 	return unless is_number($v, "real", $strict, 0,  1);
-	if(defined $a) {
-		if(is_number($a, "real", $strict, 0,1)) {
+	if (defined $a) {
+		if (is_number($a, "real", $strict, 0,1)) {
 			# ok
 		} else {
 			fatal_error("color","bad_alpha",$a);
@@ -418,7 +424,7 @@ sub validate_hsv {
 	} elsif ( $definition =~ /hsv\s*\(\s*(.+)\s*\)/i ) {
 		@hsv = split(/\s*,\s*/,$1);
 	}
-	if(@hsv && validate_hsv_list(\@hsv,$strict)) {
+	if (@hsv && validate_hsv_list(\@hsv,$strict)) {
 		return @hsv;
 	}
 	fatal_error("color","malformed_hsv",join(",",@hsv)) if $strict;
@@ -429,12 +435,12 @@ sub validate_rgb {
 	my ($definition,$strict) = @_;
 	$strict = 1 if ! defined $strict;
 	my @rgb;
-	if( ref $definition eq "ARRAY") {
+	if ( ref $definition eq "ARRAY") {
 		@rgb = @$definition;
 	} elsif ( $definition =~ /,/ ) {
 		@rgb = split(/\s*,\s*/,$definition);
 	}
-	if(@rgb && validate_rgb_list(\@rgb,$strict)) {
+	if (@rgb && validate_rgb_list(\@rgb,$strict)) {
 		return @rgb;
 	}
 	fatal_error("color","malformed_rgb",join(",",@rgb)) if $strict;
@@ -455,24 +461,24 @@ sub rgb_color {
   #confess if ! defined $color;
   return undef if ! defined $color;
   if ( $color =~ /(.+)_a(\d+)/ ) {
-      my $color_root = $1;
-      return rgb_color($color_root);
+		my $color_root = $1;
+		return rgb_color($color_root);
   } else {
-      return undef unless defined $color;
-      my @rgb;
-      if( defined $COLORS->{$color} ) {
-	  @rgb = $IM->rgb( $COLORS->{$color} );
-      } else {
-	  my $cnew = fetch_color( $color, $COLORS, $IM );
-	  @rgb = $IM->rgb( $cnew );
-      }
-      return @rgb;
-      my $colordef  = $COLORS->{$color};
-      if ($COLORS->{$colordef}) {
-	  return rgb_color($colordef);
-      }
-      @rgb = split( $COMMA, $colordef );
-      return @rgb;
+		return undef unless defined $color;
+		my @rgb;
+		if ( defined $COLORS->{$color} ) {
+			@rgb = $IM->rgb( $COLORS->{$color} );
+		} else {
+			my $cnew = fetch_color( $color, $COLORS, $IM );
+			@rgb = $IM->rgb( $cnew );
+		}
+		return @rgb;
+		my $colordef  = $COLORS->{$color};
+		if ($COLORS->{$colordef}) {
+			return rgb_color($colordef);
+		}
+		@rgb = split( $COMMA, $colordef );
+		return @rgb;
   }
 }
 
@@ -537,10 +543,10 @@ sub fetch_color {
 	$color_table ||= $COLORS;
 	$im          ||= $IM;
 	start_timer("colorfetch");
-	if(defined fetch_conf("randomcolor") && ! grep($color_name eq $_, split(",",fetch_conf("randomcolor")))) {
+	if (defined fetch_conf("randomcolor") && ! grep($color_name eq $_, split(",",fetch_conf("randomcolor")))) {
 		my @rgb = map { int rand(256) } (0,1,2);
 		my $color_name = join(",",@rgb);
-		if(defined $color_table->{$color_name}) {
+		if (defined $color_table->{$color_name}) {
 			stop_timer("colorfetch");
 			return $color_table->{$color_name};
 		} else {
@@ -560,7 +566,7 @@ sub fetch_color {
 		return $COLORS->{lc $color_name};
 	} elsif (my @rgb = validate_rgb($color_name,0)) {
 		my $color = rgb_to_color_name(@rgb,1);
-		if(defined $color) {
+		if (defined $color) {
 			stop_timer("colorfetch");
 			return $color_table->{$color};
 		} else {
@@ -586,14 +592,40 @@ sub fetch_color {
 #
 # Antialiasing in GD works only for opaque colors.
 sub aa_color {
-    my ($color_name,$im,$imcolors) = @_;
-    my $color = fetch_color($color_name,$imcolors);
-    if (not_defined_or_one(fetch_conf("anti_aliasing")) && rgb_color_opacity($color_name) == 1) {
-	$im->setAntiAliased($color);
-	return gdAntiAliased;
-    } else {
-	$color;
-    }
+	my ($color_name,$im,$imcolors) = @_;
+	my $color = fetch_color($color_name,$imcolors);
+	if (not_defined_or_one(fetch_conf("anti_aliasing")) && rgb_color_opacity($color_name) == 1) {
+		$im->setAntiAliased($color);
+		return gdAntiAliased;
+	} else {
+		$color;
+	}
+}
+
+sub find_transparent {
+	my @rgb = (0,0,0);
+	my $idx = 0;
+	my $color;
+	do {
+		$rgb[ $idx % 3]++;
+		$idx++;
+		eval { $color = rgb_to_color_name(@rgb,1) };
+	} while ($color);
+	return @rgb;
+}
+
+sub color_to_list {
+	my $color = shift;
+	my @color_names = split(/[\s+,]+/,$color);
+	my @colors;
+	for my $color_name (@color_names) {
+		if (ref $COLORS->{$color_name} eq "ARRAY") {
+	    push @colors, @{$COLORS->{$color_name}};
+		} else {
+	    push @colors, $color_name;
+		}
+	}
+	return @colors;
 }
 
 1;
