@@ -51,6 +51,7 @@ our @EXPORT = qw(
 									printwarning
 									printinfo
 									printinfof
+									printinfoq
 									printout
 
 									debug_or_group
@@ -237,17 +238,16 @@ sub format_timer {
 	sub debug_or_group {
 		my $group  = shift;
 		confess "No debug group defined." if ! defined $group;
-		# initialize debug group lookup table
+		# initialize debug group lookup table, if it has not been
+		# previously initialized or if the debug_group value is different
 		my $key = $Circos::Configuration::CONF{"debug_group"};
-		if ($key)	{
-			if (! $prevkey || $key ne $prevkey)	{
-				$lookup = {};		
-				for my $g (split($COMMA,$key)) {
-					$lookup->{$g} = 1;
-				}
+		if (! defined $prevkey || $key ne $prevkey)	{
+			$lookup = {};
+			for my $g (split($COMMA,$key)) {
+				$lookup->{$g} = 1;
 			}
-			$prevkey = $key;
 		}
+		$prevkey = $key;
 		return if ! $lookup;
 		return 1 if $lookup->{_all};
 		# groups for which reporting is asked for
@@ -290,6 +290,12 @@ sub printinfo {
 	}
 }
 
+# -------------------------------------------------------------------
+sub printinfoq {
+	printinfo(@_);
+	exit;
+}
+
 # -------------------------------------------------------------------
 sub printinfof {
 	if (! @_) {
