@@ -73,6 +73,7 @@ $ERROR{color} =
 
 $ERROR{ideogram} =
 {
+    max_number => "You have asked to draw [%d] ideograms, but the maximum is currently set at [%d]. To increase this number change max_ideograms in etc/housekeeping.conf. Keep in mind that drawing that many ideograms may create an image that is too busy and uninterpretable.",
     use_undefined => "Entry in 'chromosomes' parameter [%s] mentions chromosome [%s] which is not defined the karyotype file. Make sure that list_record_delim and list_field_delim are defined (see etc/housekeeping.conf) in order for the 'chromosomes' parameter to be parsed correctly.",
     multiple_start_anchors => "Only one chromosome order group can have a start '^' anchor",
     multiple_end_anchors => "Only one chromosome order group can have an end '\$' anchor",
@@ -90,10 +91,13 @@ $ERROR{ideogram} =
     bad_scaled_position => "Could not correctly apply scaling to find pixel position for ideogram [%s] position [%s].",
 		bad_spacing_scale => "The relative_scale_spacing parameter had a value that could not be understood [%s]. Options are min|max|average|mode for spacing based on scale statistics of all ideograms and minadj|maxadj|averageadj|modeadj if you wish to consider only ideograms flanking the space.\n\nYou can also specify the scale using a positive floating point value.",
 		bad_relative_scale => "Relative scale for a chromosome must be in the range (0,1). The scale value is meant to represent the fraction of the circle occupied by the ideogram. Your value [%s] for ideogram [%s, tag %s] is out of bounds.\nIf you want to make all ideograms the same physical size on the figure, set the same relative scale to each. For example, use\n\n chromosomes_scale = /./=0.1r",
+ undefined_axis_break_style => "You asked to use axis break style [%s] but this style is not defined. Please make sure you have an <break_style %s> block in the <ideogram><spacing> block.",
 };
 
+
 $ERROR{rule} = 
 	{
+	 cannot_undefine => "You tried to set parameter [%s] to 'undef'. You cannot undefine the position of a data point.",
 	 no_condition_no_flow => "This rule has neither a 'condition' nor a 'flow' parameter. One or both are required.\nIf you want the rule to trigger for each element, set condition = 1\nIf you want the rule to short circuit all downstream rules, set flow = stop\n\n %s",
 	 bad_flow => "Cannot understand the flow [%s] for the rule tagged [%s]",
 	 bad_tag => "You tried to goto a rule with tag [%s] but no such rule could be found.",
@@ -116,18 +120,24 @@ $ERROR{pattern} =
 
 $ERROR{configuration} = 
   {
+	 no_such_conf_item => "You attempted to reference a configuration parameter using %s, but no parameter was found at the configuration file position %s",
+   no_housekeeping => "You did not include the etc/housekeeping.conf file in your configuration file. This file contains many important system parameters and must be included in each Circos configuration.\n\nTo do so, use\n\n  <<include etc/housekeeping.conf>>\n\nin your main configuration file. For an example, see\n\n  http://www.circos.ca/documentation/tutorials/quick_guide/hello_world/configuration\n\nIf you have made other provisions to include these parameters, make sure you have\n\n  housekeeping = yes\n\ndefined to skip this check.",
    multi_word_key => "Your parameter [%s] contains a white space. This is not allowed. You either forgot a '=' in assignment (e.g. 'red 255,0,0' vs 'red = 255,0,0') or used a multi-word parameter name\n (e.g. 'my red = 255,0,0' vs 'my_red = 255,0,0'",
    missing => "file(error/configuration.missing.txt)",
    bad_parameter_type => "You attempted to reference a configuration parameter group with name [%s], but it is not defined",
    defined_twice => "Parameter [%s] of type [%s] is defined twice. Some parameters can have multiple values, but not this one.",
    multiple_defn_in_list => "Configuration value [%s] defines parameter [%s] more than once. This is not allowed.",
-   multivalue => "Configuration parameter [%s] has been defined more than once in the block shown above, and has been interpreted as a list. This is not allowed. Did you forget to comment out an old value of the parameter?",
+   multivalue => "Configuration parameter [%s] in parent block [%s] has been defined more than once in the block shown above, and has been interpreted as a list. This is not allowed. Did you forget to comment out an old value of the parameter?",
    unsupported_parameter => "Parameter [%s] of type [%s] is not supported.",
    bad_pointer => "Problem with variable lookup in configuration file. You referenced variable [%s] (seen as %s) in another parameter, but this variable is not defined.",
    no_karyotype => "You did not define a karyotype file. Are you sure your configuration file is well formed?",
 	 no_counter => "No such counter [%s]",
 	 cannot_find_include => "Error parsing the configuration file. You used an <<include FILE>> directive, but the FILE could not be found. This FILE is interpreted relative to the configuration file in which the <<include>> directive is used. Circos lookd for the file in these directories\n\n%s\n\nThe Config::General module reported the error\n\n%s",
 	 cannot_parse_file => "Error parsing the configuration file. The Config::General module reported the error\n\n%s",
+	 bad_var_value => "Could not parse variable/value pair [%s] using the delimiter regular expression [%s].",
+	 parampath_missing => "You tried to set the parameter [%s] to value [%s] using -param, but the block [%s] does not exist in the configuration file.",
+	 parampath_nothash => "You tried to set the parameter [%s] to value [%s] using -param, but the block [%s] is not unique.",
+	 undefined_parameter => "The required parameter [%s] for [%s] block was not defined",
   };
 
 $ERROR{font} = 
@@ -140,6 +150,11 @@ $ERROR{font} =
     no_ttf  => "There was a problem with True Type font support. Circos could not render text from the font file\n  %s\nPlease check that gd (system graphics library) and GD (Perl's interface to gd) are compiled with True Type support.\nOn UNIX systems, try\n  gdlib-config --all\nand look for GD_FREETYPE in the 'features' line and -lfreetype in the 'libs' line. If these are there, it's likely that your Perl GD module needs recompiling.\nFor help in installing libgd and/or GD, see\n  http://www.perlmonks.org/?node_id=621579",
 };
 
+$ERROR{svg} =
+	{
+	 no_such_tag => "No such SVG tag [%s] is defined.",
+};
+
 $ERROR{argument} =
 {
     list_size => "Function [%s] in package [%s] expected a list of size [%d] but only saw [%d] elements.",
@@ -158,7 +173,7 @@ $ERROR{graphics} =
 
 $ERROR{rules} =
 	{
-	 no_such_field => "You set up a rule [%s] that uses the parsable field [%s] but the data point you are testing does not have this field. If you want Circos to skip missing fields, set skip_missing_expression_vars=no in etc/housekeeping.conf\n\n%s.",
+	 no_such_field => "You set up a rule [%s] that uses the parsable field [%s] but the data point you are testing does not have this field. If you want Circos to skip missing fields, set skip_missing_expression_vars=yes in etc/housekeeping.conf\n\n%s.",
 	 no_field_value => "You set up a rule [%s] that uses the parsable field [%s], but this field has no associated value.",
 	 wrong_num_elements => "You set up a rule [%s] that uses the parsable field [%s] but the data point you are testing does not have [%d] elements.",
 	 parse_error => "There was a problem evaluating the string [%s] as code (error: %s)",
@@ -175,7 +190,7 @@ $ERROR{rules} =
 $ERROR{io} =
 {
     temp_dir_not_created => "Attempted to automatically determine a directory for temporary files, but failed. Please set the %s parameter to define it.",
-    cannot_write => "Cannot open file [%s] for writing %s. (error %s)",
+    cannot_write => "Cannot open file [%s] for writing %s. Make sure that the directory for this file exists (Circos will not create directories) and is writeable. (error %s)",
     cannot_read => "Cannot read file [%s] for reading %s. (error %s)",
     cannot_find => "Cannot guess the location of file [%s]. Tried to look in the following directories\n%s",
 };
@@ -199,6 +214,7 @@ $ERROR{warning} =
 
 $ERROR{track} =
 {
+ max_number => "You have asked to draw [%d] data points in a [%s] track from file\n\n %s\n\nbut the maximum is currently set at [%d]. To increase this number change max_points_per_track in etc/housekeeping.conf. Keep in mind that drawing that many data points may create an image that is too busy and uninterpretable.",
  min_larger_than_max => "Plot min value [%f] is larger than max [%f]",
  start_larger_than_end => "Input data line in file [%s] for track type [%s] has start position [%s] greater than end position [%s].",
  bad_type => "Track type [%s] is not a valid type. Choose one of [%s]. The offending track was\n\n %s",
@@ -210,6 +226,7 @@ $ERROR{track} =
 
 $ERROR{links} = 
 {
+    max_number => "You have asked to draw [%d] links from file\n\n %s\n\nbut the maximum is currently set at [%d]. To increase this number change max_links in etc/housekeeping.conf. Keep in mind that drawing that many links may create an image that is too busy and uninterpretable.",
     duplicate_names => "Multiple link data sets with name [%s] are defined. This is not supported.",
     single_entry => "Link [%s] in file [%s] has a single positional entry. A link must have two entries - a start and an end.\n\n %s",
     too_many_entries => "Link [%s] in file [%s] has [%s] coordinates. A link must have two coordinates - a start and an end.\n\n %s",
@@ -218,6 +235,11 @@ $ERROR{links} =
 
 };
 
+$ERROR{module} = 
+{
+   missing => "You are missing the module [%s]. Use CPAN to install it as described in this tutorial\n\nhttp://www.circos.ca/documentation/tutorials/configuration/perl_and_modules",
+};
+
 $ERROR{map} = 
 {
     url_param_not_set => "You have tried to use the URL [%s] for an image map, but the parameter in the url [%s] has no value defined for this data point or data set.\nTo make this error go away, either (a) define the parameter, (b) set\n image_map_missing_parameter = blank\nto remove the undefined parameter from the image element, or (c) set\n image_map_missing_parameter = removeurl\nto remove the URL from the image element.",
@@ -225,7 +247,8 @@ $ERROR{map} =
 
 $ERROR{data} =
 {
-    bad_karyotype_format => "Could not parse line from karyotype file [%s].\n\n%s",
+    bad_karyotype_format => "Could not parse line from karyotype file [%s].",
+    bad_karyotype_band_format => "Could not parse band line from karyotype file. The line\n\n  %s\n\nis missing fields. The correct format is\n\n  band chr_name band_name band_label start end color\n",
     malformed_span => "There was a problem initializing a span. Saw start [%s] > end [%s].",
     repeated_chr_in_karyotype => "Chromosome [%s] defined more than once in karyotype file.",
     malformed_karyotype_coordinates => "Start [%s] and/or end [%s] coordinate in karyotype file are not digits.",
@@ -236,6 +259,11 @@ $ERROR{data} =
     band_overlaps => "Band [%s] on chromosome [%s] overlaps with another band by more than [%s].",
 };
 
+$ERROR{ticks} =
+	{
+	 too_many => "You are trying to draw [%s] ticks on ideogram [%s]. This is a lot of ticks and will take a very long time. Make sure that chromosomes_units parameter is set and read this tutorial\n\nhttp://www.circos.ca/documentation/tutorials/ticks_and_labels/basics/\n\nTo change the maximum tick number, adjust 'max_ticks' in etc/housekeeping.conf",
+};
+
 $ERROR{function} =
 {
     remap_wrong_num_args => "You must pass five parameters to remap(). The syntax is\n remap(value,min,max,remap_min,remap_max)",
@@ -251,13 +279,14 @@ $ERROR{unit} =
 
 $ERROR{system} =
 {
-    bad_error_name => "What do you know - a fatal error caused by bad error handling.\nThe error category [%s] and name [%s] is invalid.",
-    missing_units_ok => "The parameter 'units_ok' is not defined (usually found in etc/housekeeping.conf). This parameter defines allowable units and is required. Set it to\n units_ok = %s",
-    missing_units_nounit => "The parameter 'units_nounit' is not defined (usually found in etc/housekeeping.conf). This parameter defines the explicit suffix for unitless quantities. Set it to\n units_nounit = %s",
-    wrong_unit => "The parameter [%s] value [%s] does not have the correct unit. Saw [%s] but expected one of [%s].",
-    undef_parameter => "The parameter [%s] was not defined. It needs to be defined and have one of these units [%s].",
-    unit_format_fail => "The unit [%s] failed format check. The list of allowable units is set by 'units_ok' (usually in etc/housekeeping.conf).",
-    bad_dimension => "Dimension [%s] is not defined in expression [%s]",
+ bad_error_name => "What do you know - a fatal error caused by bad error handling.\nThe error category [%s] and name [%s] is invalid.",
+ missing_units_ok => "The parameter 'units_ok' is not defined (usually found in etc/housekeeping.conf). This parameter defines allowable units and is required. Set it to\n units_ok = %s",
+ missing_units_nounit => "The parameter 'units_nounit' is not defined (usually found in etc/housekeeping.conf). This parameter defines the explicit suffix for unitless quantities. Set it to\n units_nounit = %s",
+ wrong_unit => "The parameter [%s] value [%s] does not have the correct unit. Saw [%s] but expected one of [%s].",
+ undef_parameter => "The parameter [%s] was not defined. It needs to be defined and have one of these units [%s].",
+ unit_format_fail => "The unit [%s] failed format check. The list of allowable units is set by 'units_ok' (usually in etc/housekeeping.conf).",
+ bad_dimension => "Dimension [%s] is not defined in expression [%s]",
+ hash_leaf_undef => "You tried to access the key [%s] of a hash, but it does not exist.",
 };
 
 $ERROR{support} =
