@@ -38,6 +38,7 @@ our @EXPORT = qw(
 );
 
 use Carp qw( carp confess croak );
+use Data::Dumper;
 use FindBin;
 use Params::Validate qw(:all);
 use Math::Round;
@@ -77,11 +78,11 @@ sub format_condition {
 
 # -------------------------------------------------------------------
 sub eval_expression {
-    my $expr = parse_expression(@_);
-    my $eval = eval format_condition($expr);
-    fatal_error("rules","parse_error",$expr,$@) if $@;
-    printdebug_group("rule","expression","[$expr]","eval",$eval);
-    return $eval;
+	my $expr = parse_expression(@_);
+	my $eval = eval format_condition($expr);
+	fatal_error("rules","parse_error",$expr,$@) if $@;
+	printdebug_group("rule","expression","[$expr]","eval",$eval);
+	return $eval;
 }
 
 # -------------------------------------------------------------------
@@ -123,33 +124,69 @@ sub parse_expression {
   # (.+?) replaced by (\w+)
   # parse _field_ and var(field)
   my $delim_rx = qr/(_(\w+)_)/;
-  my $var_rx   = qr/(var\((\w+)\))/;
-  while ( $expr =~ /$var_rx/i || $expr =~ /$delim_rx/i ) {
-		my ($template,$var) = ($1,lc $2);
+  my $var_rx   = qr/(var\(([\w|]+)\))/;
+  while ( $expr =~ /$var_rx/i || (fetch_conf("legacy_underline_expression_syntax") && $expr =~ /$delim_rx/i) ) {
+		my ($template,$var) = ($1,$2);
+		$var = lc $var unless fetch_conf("case_sensitive_parameter_names");
 		my ($varroot,$varnum);
-		if ($var =~ /^(.+?)(\d+)$/ ) {
+		if ($var =~ /^([^|]+?)(\d+)$/ ) {
 			($varroot,$varnum) = ($1,$2);
 		} else {
 			($varroot,$varnum) = ($var,undef);
 		}
+		my $vardef;
+		if($varroot =~ /(.+)[|](.+)/) {
+			($varroot,$vardef) = ($1,$2);
+		}
 		my $value = fetch_variable($datum,$expr,$varroot,$varnum,$param_path);
+		#printinfo($varroot,"num",$varnum,"def",$vardef,"value",$value);
+		if(! defined $value && defined $vardef) {
+			$value = $vardef;
+		}
 		replace_string( \$expr, $template, $value );
   }
 
   # parse functions f(var)
-  for my $f (qw(on between fromto tofrom from to )) {
+  for my $f (qw(on within between fromto tofrom from to chrlen)) {
 		# for perl 5.10 using recursive rx
 		# my $parens = qr/(\((?:[^()]++|(?-1))*+\))/;
 		# no longer using this, to make the code compatible with 5.8
 		# while( $expr =~ /($f$parens)/ ) {
-
 		while(my ($template,$arg) = extract_balanced($expr,$f,"(",")")) {
 			$template = $f . $template;
 			if($f eq "on") {
-	      my ($arg1) = split(",",$arg);
+	      my ($arg1,$arg2,$arg3) = split(",",$arg);
 	      fatal_error("rule","fn_wrong_arg",$f,$expr_orig,1) if ! defined $arg1;
-	      #printinfo($template,$arg_nested,$arg,$arg1);
-	      my $result = grep($_ =~ /^$arg1$/, map {$_->{chr}} @{$datum->{data}});
+				my $c0 = $datum->{data}[0]{chr} =~ /^$arg1/;
+				my $c1 = $datum->{data}[1]{chr} =~ /^$arg1/ if exists $datum->{data}[1];
+				if(defined $arg2 && defined $arg3) {
+					if($c0) {
+						my $d0 = span_distance($arg2,$arg3,$datum->{data}[0]{start},$datum->{data}[0]{end});
+						$c0 &&= $d0 < 0;
+					}
+					if($c1) {
+						my $d1 = span_distance($arg2,$arg3,$datum->{data}[1]{start},$datum->{data}[1]{end});
+						$c1 &&= $d1 < 0;
+					}
+				}
+				my $result = $c0 || $c1 || 0;
+	      replace_string( \$expr, $template, $result);
+			} elsif ($f eq "within") {
+	      my ($arg1,$arg2,$arg3) = split(",",$arg);
+	      fatal_error("rule","fn_wrong_arg",$f,$expr_orig,1) if ! defined $arg1;
+				my $c0 = $datum->{data}[0]{chr} =~ /^$arg1/;
+				my $c1 = $datum->{data}[1]{chr} =~ /^$arg1/ if exists $datum->{data}[1];
+				if(defined $arg2 && defined $arg3) {
+					if($c0) {
+						my $d0 = span_distance($arg2,$arg3,$datum->{data}[0]{start},$datum->{data}[0]{end});
+						$c0 &&= -$d0 == $datum->{data}[0]{set}->cardinality-1;
+					}
+					if($c1) {
+						my $d1 = span_distance($arg2,$arg3,$datum->{data}[1]{start},$datum->{data}[1]{end});
+						$c1 &&= -$d1 == $datum->{data}[0]{set}->cardinality-1;
+					}
+				}
+				my $result = $c0 || $c1 || 0;
 	      replace_string( \$expr, $template, $result);
 			} elsif ($f eq "between") {
 	      my ($arg1,$arg2) = split(",",$arg);
@@ -184,6 +221,12 @@ sub parse_expression {
 	      fatal_error("rule","fn_need_2_coord",$f,$expr_orig,$arg1,"-") if $num_coord != 2;
 	      my $result = $datum->{data}[0]{chr} =~ /^$arg1$/i;
 	      replace_string( \$expr, $template, $result || 0);
+			} elsif ($f eq "chrlen") {
+				my ($arg) = split(",",$arg);
+				$arg =~ s/[\'\"]//g;
+				my $ideograms = Circos::get_ideograms_by_name($arg);
+				my $result = $ideograms->[0]->{chrlength};
+				replace_string( \$expr, $template, $result);
 			}
 		}
   }
@@ -192,9 +235,7 @@ sub parse_expression {
 
 sub fetch_variable {
 	my ($datum,$expr,$var,$varnum,$param_path) = @_;
-	
 	my $num_coord = @{$datum->{data}};
-	
 	# If this data collection has only one data value (e.g. scatter plot)
 	# then assume that any expression without an explicit number is refering
 	# to the data point (e.g. _SIZE_ acts like _SIZE1_)
@@ -208,10 +249,13 @@ sub fetch_variable {
 		}
 	} elsif ($num_coord == 2) {
 		if(! defined $varnum) {
-	    # var(START) treated like var(START1) but only if var(START1) == var(START2)
+	    # var(START) treated like var(START1) but only if var(START1) eq var(START2)
 	    my $v1 = fetch_variable($datum,$expr,$var,1,$param_path);
 	    my $v2 = fetch_variable($datum,$expr,$var,2,$param_path);
-	    if($v1 eq $v2) {
+	    if( (! defined $v1 && ! defined $v2) || ($v1 eq $v2) ) {
+				# the only consistent conditions are
+				# - neither is defined
+				# - both are defined and equal
 				return $v1;
 	    } else {
 				fatal_error("rule","conflicting_coord",
@@ -226,14 +270,11 @@ sub fetch_variable {
 	} else {
 		fatal_error("rule","wrong_coord_num",$num_coord);
 	}
-	
+
 	my $varidx = $varnum - 1;
-	
 	my $data = $datum->{data};
 	my $value;
-	
-	$var = lc $var;
-	
+	$var = lc $var unless fetch_conf("case_sensitive_parameter_names");
 	if( exists $datum->{param}{$var} ) {
 		$value = $datum->{param}{$var};
 	} elsif ( exists $data->[$varidx]{$var} ) {
@@ -252,12 +293,12 @@ sub fetch_variable {
 		$value = $data->[0]{chr} ne $data->[1]{chr} ? 1 : 0;
 	} else {
 		if(fetch_conf("skip_missing_expression_vars")) {
-	    $value = $EMPTY_STR;
+	    $value = undef;
 		} else {
 	    fatal_error("rules","no_such_field",$expr,$var,Dumper($datum));
 		}
 	}
-	$value = Circos::unit_strip($value);
+	#$value = Circos::unit_strip($value);
 	printdebug_group("rule","found variable",$var."[$varnum]","value",$value);
 	return $value;
 }
@@ -265,11 +306,14 @@ sub fetch_variable {
 # -------------------------------------------------------------------
 sub replace_string {
   my ( $target, $source, $value ) = @_;
-  if ( $value =~ /[^0-9-.]/ && $value ne "undef" ) {
-    $$target =~ s/\Q$source\E/'$value'/g;
-  } else {
-    $$target =~ s/\Q$source\E/$value/g;
-  }
+	$value = 0 if ! defined $value;
+
+	if ( $value =~ /[^0-9-.]/ && $value ne "undef" ) {
+		$$target =~ s/\Q$source\E/'$value'/g;
+	} else {
+		$$target =~ s/\Q$source\E/$value/g;
+	}
+
 }
 
 
