@@ -36,7 +36,7 @@ use warnings;
 use base 'Exporter';
 our @EXPORT = qw(
 
-);
+							 );
 
 use Carp qw( carp confess croak );
 use Cwd;
@@ -86,110 +86,116 @@ use Circos::Utils;
 #
 
 sub read_karyotype {
-    fetch_conf("debug_validate") && validate( @_, { file => 1 } );
-    my %params = @_;
-    my $files = Circos::Configuration::make_parameter_list_array($params{file});
-    my $karyotype = {};
-    for my $file (@$files) {
-	read_karyotype_file($file,$karyotype);
-    }
-    return $karyotype;
+	fetch_conf("debug_validate") && validate( @_, { file => 1 } );
+	my %params = @_;
+	my $files = Circos::Configuration::make_parameter_list_array($params{file});
+	my $karyotype = {};
+	for my $file (@$files) {
+		read_karyotype_file($file,$karyotype);
+	}
+	return $karyotype;
 }
 
 sub read_karyotype_file {
-    my ($file,$karyotype) = @_;
-    $file = locate_file( file => $file, name=>"karyotype" );
-    my $chr_index;
-    if (! keys %$karyotype) {
-	$chr_index = 0;
-    } else {
-	my @prev_index = map { $_->{chr}{display_order} } values %$karyotype;
-	$chr_index = 1 + max @prev_index;
-    }
-    
-    my $delim_rx = fetch_conf("file_delim") || undef;
-    if ($delim_rx && fetch_conf("file_delim_collapse")) {
-	$delim_rx .= "+";
-    }
-    
-    open(F,$file) or fatal_error("io","cannot_read",$file,"karyotype",$!);
+	my ($file,$karyotype) = @_;
+	my $file_located = locate_file(file=>$file,name=>"karyotype",return_undef=>1);
+	if(! $file_located) {
+		fatal_error("io","cannot_read",$file,"karyotype",$!);
+	}
+	my $chr_index;
+	if (! keys %$karyotype) {
+		$chr_index = 0;
+	} else {
+		my @prev_index = map { $_->{chr}{display_order} } values %$karyotype;
+		$chr_index = 1 + max @prev_index;
+	}
 
-    while (<F>) {
-	chomp;
-	my $line = $_;
-	next if is_blank($line);
-	next if is_comment($line);
-	
-	my @tok = $delim_rx ? split($delim_rx,$line) : split;
-	
-	my ($field,$parent,$name,$label,$start,$end,$color,$options) = @tok;
-	
-	#fatal_error("data","bad_karyotype_format",$file,$line);
-	
-	if ( ! is_number($start,"int") || ! is_number($end,"int") ) {
+	my $delim_rx = fetch_conf("file_delim") || undef;
+	if ($delim_rx && fetch_conf("file_delim_collapse")) {
+		$delim_rx .= "+";
+	}
+
+	open(F,$file_located) or fatal_error("io","cannot_read",$file_located,"karyotype",$!);
+
+	while (<F>) {
+		chomp;
+		my $line = $_;
+		next if is_blank($line);
+		next if is_comment($line);
+
+		my @tok = $delim_rx ? split($delim_rx,$line) : split;
+
+		my ($field,$parent,$name,$label,$start,$end,$color,$options) = @tok;
+
+		#fatal_error("data","bad_karyotype_format",$file,$line);
+
+		if ( ! is_number($start,"int") || ! is_number($end,"int") ) {
 	    fatal_error("data","malformed_karyotype_coordinates",$start,$end);
-	}
-	if ( $end <= $start ) {
-	    fatal_error("data","inconsistent_karyotype_coordinates",$start,$end);
-	}
-	
-	my $set  = make_set($start,$end,norev=>1);
-	
-	# karyotype data structure is a hash with each chromosome being a value
-	# keyed by chromosome name. Bands form a list within the chromosome
-	# data structure, keyed by 'band'.
-	
-	my $data = {
-	    start   => $start,
-	    end     => $end,
-	    set     => $set,
-	    size    => $set->cardinality,
-	    name    => $name,
-	    label   => $label,
-	    parent  => $parent,
-	    color   => lc $color,
-	    options => parse_options($options)
-	};
-
-	if ( $field =~ /chr/ ) {
-	    # chromosome entries have a few additional fields
-	    # chr, scale, display_order
-	    $data->{chr}           = $name;
-	    $data->{scale}         = 1;
-	    $data->{display_order} = $chr_index++;
-	    if ( $karyotype->{$data->{chr}}{chr} ) {
-		fatal_error("data","repeated_chr_in_karyotype",$data->{chr});
-	    }
-	    # check if color override has been specified
-	    my $color_conf_str = fetch_conf("chromosomes_color") || fetch_conf("chromosome_color");
-	    if ($color_conf_str) {
-		my @color_list = @{Circos::Configuration::make_parameter_list_array($color_conf_str)};
-		for my $is_rx (1,0) {
-		    for my $color_pair (@color_list) {
-			my ($key,$value) = split(/\s*=\s*/,$color_pair);
-			my $rx           = parse_as_rx($key);
-			if ($is_rx) {
-			    next unless $rx;
-			} else {
-			    next if $rx;
-			}
-			my $match = match_string($name,$rx||$key);
-			$data->{color} = $value if $match;
-		    }
 		}
+		if ( $end <= $start ) {
+	    fatal_error("data","inconsistent_karyotype_coordinates",$start,$end);
+		}
+		if (@tok < 7) {
+	    fatal_error("data","bad_karyotype_band_format",$line);
+		}
+	
+		my $set  = make_set($start,$end,norev=>1);
+	
+		# karyotype data structure is a hash with each chromosome being a value
+		# keyed by chromosome name. Bands form a list within the chromosome
+		# data structure, keyed by 'band'.
+	
+		my $data = {
+								start   => $start,
+								end     => $end,
+								set     => $set,
+								size    => $set->cardinality,
+								name    => $name,
+								label   => $label,
+								parent  => $parent,
+								color   => lc $color,
+								options => parse_options($options)
+							 };
+
+		if ( $field =~ /chr/ ) {
+			# chromosome entries have a few additional fields
+			# chr, scale, display_order
+			$data->{chr}           = $name;
+			$data->{scale}         = 1;
+			$data->{display_order} = $chr_index++;
+			if ( $karyotype->{$data->{chr}}{chr} ) {
+				fatal_error("data","repeated_chr_in_karyotype",$data->{chr});
+			}
+			# check if color override has been specified
+			my $color_conf_str = fetch_conf("chromosomes_color") || fetch_conf("chromosome_color");
+	    if ($color_conf_str) {
+				my @color_list = @{Circos::Configuration::make_parameter_list_array($color_conf_str)};
+				for my $is_rx (1,0) {
+					for my $color_pair (@color_list) {
+						my ($key,$value) = Circos::Configuration::parse_var_value($color_pair);
+						my $rx           = parse_as_rx($key);
+						if ($is_rx) {
+							next unless $rx;
+						} else {
+							next if $rx;
+						}
+						my $match = match_string($name,$rx||$key);
+						$data->{color} = $value if $match;
+					}
+				}
 	    }
 	    # chromosome is keyed by its name
 	    $karyotype->{ $name }{chr} = $data;
-	} elsif ( $field =~ /band/ ) {
+		} elsif ( $field =~ /band/ ) {
 	    # band entries have the 'chr' key point to the name of parent chromosome
 	    $data->{chr} = $parent;
 	    push @{ $karyotype->{ $parent }{band} }, $data;
-	} else {
+		} else {
 	    # for now, die hard here. There are no other line types supported.
 	    fatal_error("data","unknown_karyotype_line",$field,$line);
+		}
 	}
-    }
-    return $karyotype;
+	return $karyotype;
 }
 
 ################################################################
@@ -243,35 +249,35 @@ sub validate_karyotype {
 # asciibetically.
 #
 sub sort_karyotype {
-    fetch_conf("debug_validate") && validate( @_, { karyotype => 1 } );
-    my %params = @_;
-    my $k = $params{karyotype};
+	fetch_conf("debug_validate") && validate( @_, { karyotype => 1 } );
+	my %params = @_;
+	my $k = $params{karyotype};
     
-    my @chrs = sort { $k->{$a}{chr}{display_order} <=> $k->{$b}{chr}{display_order} } keys %$k;
+	my @chrs = sort { $k->{$a}{chr}{display_order} <=> $k->{$b}{chr}{display_order} } keys %$k;
     
-    my @chrs_native_sort;
+	my @chrs_native_sort;
 
-    if (my @chrs_w_num = grep($_ =~ /\d/, @chrs)) {
-	# if there are any chromosomes with a number in them, place them first and
-	# sort them by the (preceeding) non-numerical string first, then the number
-	my $rxnd = qr/^(\D+)/;
-	my $rxd  = qr/(\d+)/;
-	my @chrs_for_sort;
-	for my $c (@chrs_w_num) {
+	if (my @chrs_w_num = grep($_ =~ /\d/, @chrs)) {
+		# if there are any chromosomes with a number in them, place them first and
+		# sort them by the (preceeding) non-numerical string first, then the number
+		my $rxnd = qr/^(\D+)/;
+		my $rxd  = qr/(\d+)/;
+		my @chrs_for_sort;
+		for my $c (@chrs_w_num) {
 	    my ($non_digit) = $c =~ /$rxnd/;
 	    my ($digit)     = $c =~ /$rxd/;
 	    push @chrs_for_sort, {chr       => $c,
-				  non_digit => $non_digit || $EMPTY_STR,
-				  digit     => $digit     || 0};
+														non_digit => $non_digit || $EMPTY_STR,
+														digit     => $digit     || 0};
+		}
+		push @chrs_native_sort, map { $_->{chr} } sort { ( $a->{non_digit} cmp $b->{non_digit} ) ||
+																											 ( $a->{digit} <=> $b->{digit} ) } @chrs_for_sort;
+	}
+	push @chrs_native_sort, sort { $a cmp $b } grep($_ !~ /\d/, @chrs);
+	for my $i (0..@chrs_native_sort-1) {
+		my $chr = $chrs_native_sort[$i];
+		$k->{$chr}{chr}{sort_idx} = $i;
 	}
-	push @chrs_native_sort, map { $_->{chr} } sort { ( $a->{non_digit} cmp $b->{non_digit} ) ||
-							     ( $a->{digit} <=> $b->{digit} ) } @chrs_for_sort;
-    }
-    push @chrs_native_sort, sort { $a cmp $b } grep($_ !~ /\d/, @chrs);
-    for my $i (0..@chrs_native_sort-1) {
-	my $chr = $chrs_native_sort[$i];
-	$k->{$chr}{chr}{sort_idx} = $i;
-    }
 }
 
 1;
