@@ -72,12 +72,15 @@ is_integer
 is_blank
 is_comment
 make_set
+span_distance
 show
 hide
 use_set
 true_or_yes
 false_or_no
 parse_options
+get_hash_leaf
+log10
 );
 
 use Carp qw( carp confess croak );
@@ -116,73 +119,75 @@ sub make_list {
 }
 
 sub round_custom {
-    my ($x,$round_type) = @_;
-    if(! defined $round_type) {
-	return int $x;
-    } elsif ($round_type eq "round") {
-	return round $x;
-    } elsif ($round_type eq "floor") {
-	return floor $x;
-    } elsif ($round_type eq "ceil") {
-	return ceil $x;
-    }
+	my ($x,$round_type) = @_;
+	if (! defined $round_type) {
+		return int $x;
+	} elsif ($round_type eq "round") {
+		return round $x;
+	} elsif ($round_type eq "floor") {
+		return floor $x;
+	} elsif ($round_type eq "ceil") {
+		return ceil $x;
+	}
 }
 #################################################################
 # 
 sub remap {
-    my ($value,$min,$max,$remap_min,$remap_max) = @_;
-    if(! defined $value ||
-       ! defined $min ||
-       ! defined $max ||
-       ! defined $remap_min ||
-       ! defined $remap_max) {
-	fatal_error("function","remap_wrong_num_args");
-    }
-    return $remap_min if $value <= $min;
-    return $remap_max if $value >= $max;
 
-    if($min == $max) {
-	if($remap_min == $remap_max) {
-	    return $remap_min;
-	} else {
-	    fatal_error("function","remap_min_max",$min,$max,$remap_min,$remap_max);
+	my ($value,$min,$max,$remap_min,$remap_max) = @_;
+	if (! defined $value ||
+			! defined $min ||
+			! defined $max ||
+			! defined $remap_min ||
+			! defined $remap_max) {
+		fatal_error("function","remap_wrong_num_args");
 	}
-    }
-    my $f = ( $value - $min ) / ( $max - $min );
-    my $value_remap = $remap_min + $f * ($remap_max - $remap_min);
-    #printinfo($value,$min,$max,$remap_min,$remap_max,$value_remap);
-    return $value_remap;
+
+	return $remap_min if $value <= $min;
+	return $remap_max if $value >= $max;
+
+	if ($min == $max) {
+		if ($remap_min == $remap_max) {
+	    return $remap_min;
+		} else {
+	    fatal_error("function","remap_min_max",$min,$max,$remap_min,$remap_max);
+		}
+	}
+	my $f = ( $value - $min ) / ( $max - $min );
+	my $value_remap = $remap_min + $f * ($remap_max - $remap_min);
+	#printinfo($value,$min,$max,$remap_min,$remap_max,$value_remap);
+	return $value_remap;
 }
 
 sub pairwise_or {
-    my ($a,$b,$x,$y) = @_;
-    if(! defined $a || ! defined $b || ! defined $x || ! defined $y) {
-	fatal_error("function","pairwise","pairwise_or",$a,$b,$x,$y);
-    } else {
-	return ($a eq $x && $b eq $y) || ($a eq $y && $b eq $x)
-    }
+	my ($a,$b,$x,$y) = @_;
+	if (! defined $a || ! defined $b || ! defined $x || ! defined $y) {
+		fatal_error("function","pairwise","pairwise_or",$a,$b,$x,$y);
+	} else {
+		return ($a eq $x && $b eq $y) || ($a eq $y && $b eq $x)
+	}
 }
 
 sub pairwise_and {
-    my ($a,$b,$x,$y) = @_;
-    if(! defined $a || ! defined $b || ! defined $x || ! defined $y) {
-	fatal_error("function","pairwise","pairwise_and",$a,$b,$x,$y);
-    } else {
-	return $a eq $x && $b eq $y;
-    }
+	my ($a,$b,$x,$y) = @_;
+	if (! defined $a || ! defined $b || ! defined $x || ! defined $y) {
+		fatal_error("function","pairwise","pairwise_and",$a,$b,$x,$y);
+	} else {
+		return $a eq $x && $b eq $y;
+	}
 }
 
 sub remap_int {
-    return int remap(@_);
+	return int remap(@_);
 }
 
 sub remap_round {
-    return round remap(@_);
+	return round remap(@_);
 }
 
 sub str_to_list {
-    my $str = shift;
-    return split(/\s*,\s*/, $str);
+	my $str = shift;
+	return split(/\s*,\s*/, $str);
 }
 
 # -------------------------------------------------------------------
@@ -193,7 +198,7 @@ sub sample_list {
 	# The results are sorted based on capture buffers from the regular expression.
 	my ($rx,$list) = @_;
 	my $rev = 0;
-	if($rx =~ /rev\((.+)\)/) {
+	if ($rx =~ /rev\((.+)\)/) {
 		$rx  = $1;
 		$rev = 1;
 	}
@@ -201,7 +206,7 @@ sub sample_list {
 	return undef if ! $list;
 	my @matches;
 	for my $item (@$list) {
-		if($item =~ /^$rx$/) {
+		if ($item =~ /^$rx$/) {
 	    # pull out captured strings
 	    my @captures = ();
 	    for my $i (1..@+ - 1) {
@@ -210,9 +215,9 @@ sub sample_list {
 	    }
 	    push @matches, {item=>$item,captures=>\@captures};
 		}
-    }
+	}
 	my @result = map { $_->{item} } sort { compare_strs($a->{captures},$b->{captures}) } @matches;
-	if($rev) {
+	if ($rev) {
 		return reverse @result;
 	} else {
 		return @result;
@@ -221,17 +226,18 @@ sub sample_list {
 
 sub parse_as_rx {
 	my $rx = shift;
-    if ($rx =~ /^-?\/(.+)\/$/) {
-			return qr/$1/;
-    } else {
-			return;
-    }
+	return if ! defined $rx;
+	if ($rx =~ /^\s*-?\/(.+)\/\s*$/) {
+		return qr/$1/;
+	} else {
+		return;
+	}
 }
 
 sub is_num_equal {
-    my ($x,$y) = @_;
-    return unless defined $x && defined $y;
-    return $x == $y;
+	my ($x,$y) = @_;
+	return unless defined $x && defined $y;
+	return $x == $y;
 }
 
 sub is_num_notequal {
@@ -248,7 +254,7 @@ sub match_string {
 	my ($str,$rx) = @_;
 	return unless defined $str;
 	return unless defined $rx;
-	if(ref($rx)) {
+	if (ref($rx)) {
 		return $str =~ /$rx/;
 	} else {
 		return $str eq $rx;
@@ -257,43 +263,43 @@ sub match_string {
 
 # -------------------------------------------------------------------
 sub compare_strs {
-    my ($list1,$list2) = @_;
-    my $result = 0;
-    for my $i (0..@$list1-1) {
-	return $result if ! defined $list1->[$i] || ! defined $list2->[$i];
-	$result ||= compare_str($list1->[$i],$list2->[$i]);
-    }
-    return $result;
+	my ($list1,$list2) = @_;
+	my $result = 0;
+	for my $i (0..@$list1-1) {
+		return $result if ! defined $list1->[$i] || ! defined $list2->[$i];
+		$result ||= compare_str($list1->[$i],$list2->[$i]);
+	}
+	return $result;
 }
 
 # -------------------------------------------------------------------
 sub extract_number {
-    my $str = shift;
-    if($str =~ /0*(\d+)/) {
-	return $1;
-    } else {
-	return "";
-    }
+	my $str = shift;
+	if ($str =~ /0*(\d+)/) {
+		return $1;
+	} else {
+		return "";
+	}
 }
 
 # -------------------------------------------------------------------
 sub compare_str {
-    my ($x,$y) = @_;
-    if( $x =~ /$RE{num}{real}/ && $y =~ /$RE{num}{real}/ ) {
-	$x =~ s/^0*//;
-	$y =~ s/^0*//;
-	$x ||= 0;
-	$y ||= 0;
-	return $x <=> $y;
-    } else {
-	return $x cmp $y;
-    }
+	my ($x,$y) = @_;
+	if ( $x =~ /$RE{num}{real}/ && $y =~ /$RE{num}{real}/ ) {
+		$x =~ s/^0*//;
+		$y =~ s/^0*//;
+		$x ||= 0;
+		$y ||= 0;
+		return $x <=> $y;
+	} else {
+		return $x cmp $y;
+	}
 }
 
 # -------------------------------------------------------------------
 sub round_up {
 	my $value = shift;
-	if($value > int($value)) {
+	if ($value > int($value)) {
 		return 1 + int($value);
 	} else {
 		return $value;
@@ -302,10 +308,10 @@ sub round_up {
 
 # -------------------------------------------------------------------
 sub put_between {
-    my ($x,$min,$max) = @_;
-    return $min if $x < $min;
-    return $max if $x > $max;
-    return $x;
+	my ($x,$min,$max) = @_;
+	return $min if $x < $min;
+	return $max if $x > $max;
+	return $x;
 }
 
 ################################################################
@@ -315,18 +321,18 @@ sub is_number {
 	$strict = 1 if ! defined $strict;
 	my $pass = 1;
 	eval {
-		if($x !~ /^$RE{num}{$rxtype}$/) {
+		if ($x !~ /^$RE{num}{$rxtype}$/) {
 			$pass = 0;
 			fatal_error("parsedata","bad_number",$x,$rxtype) if $strict;
 		}
 	};
-	if($@) {
+	if ($@) {
 		fatal_error("parsedata","no_such_re",$x,$rxtype,$@);
 	}
 	return if ! $pass;
-	if( (defined $min && $x < $min)
-			||
-			(defined $max && $x > $max)) {
+	if ( (defined $min && $x < $min)
+			 ||
+			 (defined $max && $x > $max)) {
 		my $min_text = defined $min ? $min : "(";
 		my $max_text = defined $max ? $max : ")";
 		$pass = 0;
@@ -369,26 +375,30 @@ sub seek_parameter {
   # seek_parameter("x|y",$hash,$anotherhash);
   my ( $param_name, @data_structs ) = @_;
   my @target_string = split( /\|/, $param_name );
+	my $not_def_ok = 1;
   start_timer("parameter_seek");
   for my $str (@target_string) {
     for my $struct (@data_structs) {
       if ( ref($struct) eq "ARRAY" ) {
 				for my $substruct (@$struct) {
-					if(exists $substruct->{param} && defined $substruct->{param}{$str}) {
+					if (exists $substruct->{param} &&
+							exists $substruct->{param}{$str} && ($not_def_ok || defined $substruct->{param}{$str})) {
 						stop_timer("parameter_seek");
 						return $substruct->{param}{$str};
 					}
-					if(exists $substruct->{$str}  && defined $substruct->{$str}) {
+					if (exists $substruct->{$str}  && ($not_def_ok || defined $substruct->{$str})) {
 						stop_timer("parameter_seek");
 						return $substruct->{$str};
 					}
 				}
       } elsif ( ref($struct) eq "HASH" ) {
-				if(exists $struct->{param} && defined $struct->{param}{$str}) {
+				if (exists $struct->{param} &&
+						exists $struct->{param}{$str} &&
+						($not_def_ok || defined $struct->{param}{$str})) {
 					stop_timer("parameter_seek");
 					return $struct->{param}{$str};
 				}
-				if(exists $struct->{$str} && defined $struct->{$str}) {
+				if (exists $struct->{$str} && ($not_def_ok || defined $struct->{$str})) {
 					stop_timer("parameter_seek");
 					return $struct->{$str};
 				}
@@ -403,81 +413,83 @@ sub seek_parameter {
 }
 
 sub is_hidden {
-    my @datapath   = @_;
-    my $show_state = seek_parameter("show", @datapath);
-    return defined_and_zero($show_state);
+	my @datapath   = @_;
+	my $show_state = seek_parameter("show", @datapath);
+	return defined_and_zero($show_state);
 }
 
 sub get_file_annotation {
-    my %params = @_;
-    my $file   = $params{file};
-    my ($filename,@annot) = split(",",$file);
-    return join(",",@annot);
+	my %params = @_;
+	my $file   = $params{file};
+	my ($filename,@annot) = split(",",$file);
+	return join(",",@annot);
 }
 
 # -------------------------------------------------------------------
 sub locate_file {
-    my %params;
-    if($Circos::Configuration::CONF{debug_validate}) {
-	%params = validate(@_,{ 
-	    file => 1, 
-	    name => 0,
-	    path => { type => ARRAYREF | UNDEF, optional => 1 },
-	    return_undef => 0 
-			   });
-    } else {
-	%params = @_;
-    }
-    # look for the file in various directories
-    my @dir_1 = (getcwd,$FindBin::RealBin);
-    my @dir_2 = qw(. .. ../..);
-    my @dir_3 = qw(. etc data);
+	my %params;
+	if ($Circos::Configuration::CONF{debug_validate}) {
+		%params = validate(@_,{ 
+													 file => 1, 
+													 name => 0,
+													 path => { type => ARRAYREF | UNDEF, optional => 1 },
+													 return_undef => 0 
+													});
+	} else {
+		%params = @_;
+	}
 
-    my $file   = $params{file};
-    # remove any comma-delimited elements from the file
-    $file =~ s/,.*//;
-    printdebug_group("io","locating file",$file,"role",$params{name});
-    
-    if(! defined $file) {
-	confess "Attempted to locate an undefined file name for [$params{name}]";
-    }
-    
-    my @path;
-    if($file =~ /^\//) {
-	@path = ($EMPTY_STR);
-    } else {
-	# first add any custom path directories
-	push @path, @{$params{path}} if defined $params{path};
-	if ( my $path_list = Circos::Configuration::fetch_conf("data_path") ) {
+	# look for the file in various directories
+	# v0.63 added configuration directory to dir_1
+	my @dir_1 = grep($_,getcwd,Circos::Configuration::fetch_conf("configdir"),$FindBin::RealBin);
+	my @dir_2 = qw(. .. ../.. ../../..);
+	my @dir_3 = qw(. etc data);
+
+	my $file   = $params{file};
+	# remove any comma-delimited elements from the file
+	$file =~ s/,.*//;
+	printdebug_group("io","locating file",$file,"role",$params{name});
+
+	if (! defined $file) {
+		confess "Attempted to locate an undefined file name for [$params{name}]";
+	}
+
+	my @path;
+	if (file_name_is_absolute($file)) {
+		@path = ($EMPTY_STR);
+	} else {
+		# first add any custom path directories
+		push @path, @{$params{path}} if defined $params{path};
+		if ( my $path_list = Circos::Configuration::fetch_conf("data_path") ) {
 	    push @path, split($COMMA,$path_list);
-	}
-	# now the default locations
-	for my $d1 (@dir_1) {
-	    for my $d2 (@dir_2) {
-		for my $d3 (@dir_3) {
-		    push @path, catfile($d1,$d2,$d3);
 		}
+		# now the default locations
+		for my $d1 (@dir_1) {
+	    for my $d2 (@dir_2) {
+				for my $d3 (@dir_3) {
+					push @path, catfile($d1,$d2,$d3);
+				}
 	    }
+		}
 	}
-    }
-    printdebug_group("io","trying path",@path);
-    for my $path (@path) {
-	my $file_path = catfile($path,$file);
-	printdebug_group("io","trying $file_path");
-	if ( -e $file_path) {
-	    if(! -r $file_path) {
-		fatal_error("io","cannot_read",$file,"with locate_file",$!);
+	printdebug_group("io","trying path",@path);
+	for my $path (@path) {
+		my $file_path = $path ? catfile($path,$file) : $file;
+		printdebug_group("io","trying $file_path");
+		if ( -e $file_path) {
+	    if (! -r $file_path) {
+				fatal_error("io","cannot_read",$file,"with locate_file",$!);
 	    } else {
-		printdebug_group("io","$file found in $file_path");
-		return $file_path;
+				printdebug_group("io","$file found in $file_path");
+				return $file_path;
 	    }
+		}
+	}
+	if ( $params{return_undef} ) {
+		return undef;
+	} else {
+		fatal_error("io","cannot_find",$file,join("\n",map { " $_" } @path));
 	}
-    }
-    if ( $params{return_undef} ) {
-	return undef;
-    } else {
-	fatal_error("io","cannot_find",$file,join("\n",map { " $_" } @path));
-    }
 }
 
 # -------------------------------------------------------------------
@@ -494,12 +506,12 @@ sub add_thousands_separator {
 
 # -------------------------------------------------------------------
 sub not_defined_or_one {
-    return !defined $_[0] || $_[0];
+	return !defined $_[0] || $_[0];
 }
 
 # -------------------------------------------------------------------
 sub defined_and_zero {
-    return defined $_[0] && !$_[0];
+	return defined $_[0] && !$_[0];
 }
 
 sub first_defined {
@@ -520,14 +532,14 @@ sub is_in_list {
 
 # -------------------------------------------------------------------
 sub current_function {
-    my ($package,$filename,$line,$function) = caller(1);
-    $function =~ s/.*:://g;
-    return $function;
+	my ($package,$filename,$line,$function) = caller(1);
+	$function =~ s/.*:://g;
+	return $function;
 }
 
 sub current_package {
-    my ($package,$filename,$line,$function) = caller(1);
-    return $package;
+	my ($package,$filename,$line,$function) = caller(1);
+	return $package;
 }
 
 # -------------------------------------------------------------------
@@ -536,17 +548,17 @@ sub parse_csv {
 	my @elems  = split("",$str);
 	my @params;
 	my $paren_level;
-	while(scalar @elems) {
+	while (scalar @elems) {
 		my $elem = shift @elems;
-		if($elem eq "(") {
+		if ($elem eq "(") {
 			$paren_level++;
-		} elsif($elem eq ")") {
+		} elsif ($elem eq ")") {
 			$paren_level--;
-		} elsif($elem ne ",") {
+		} elsif ($elem ne ",") {
 			push @params, "" if ! @params;
 			$params[-1] .= $elem;
-		} elsif($elem eq ",") {
-			if(! $paren_level) {
+		} elsif ($elem eq ",") {
+			if (! $paren_level) {
 				push @params, "";
 			} else {
 				push @params, "" if ! @params;
@@ -554,25 +566,85 @@ sub parse_csv {
 			}
 		}
 	}
-	if($paren_level) {
+	if ($paren_level) {
 		fatal_error("parsedata","bad_csv",$str,$paren_level);
 	}
 	return @params;
 }
 
 sub remove_undef_keys {
-    my %x = @_;
-    return map { ($_,$x{$_}) } grep(defined $x{$_}, keys %x);
+	my %x = @_;
+	return map { ($_,$x{$_}) } grep(defined $x{$_}, keys %x);
+}
+
+# return the distance between the span
+# [x1,y1] and [x2,y2]
+# if the spans overlap, the distance is negative
+sub span_distance {
+  my ($x1,$y1,$x2,$y2) = @_;
+  # flip the coordinates if they are reversed
+  ($x1,$y1) = ($y1,$x1) if $x1 > $y1;
+  ($x2,$y2) = ($y2,$x2) if $x2 > $y2;
+  # flip intervals so that x1,y1 is always to the left
+  ($x1,$y1,$x2,$y2) = ($x2,$y2,$x1,$y1) if ($x1 > $x2);
+  my $d;
+  if ($x2 >= $y1) {
+    # x1 y1
+    # -----  
+    #        x2  y2
+    #        ------
+    $d = $x2 - $y1;
+  } else {
+    if ($y2 >= $y1) {
+      # x1     y1
+      # ---------
+      #     x2    y2
+      #     --------
+      $d = -($y1 - $x2);
+    } else {
+      # x1     y1
+      # ---------
+      #   x2  y2
+      #   ------
+      $d = -($y2-$x2);
+    }
+  }
+  die "did not calculate distance between intervals [$x1,$y1] and [$x2,$y2] correctly." unless defined $d;
+  return $d;
+
+  # test
+  for my $i (0..100000) {
+    my @coords = map { sprintf("%.1f",100*rand()) } (0..3);
+    my $s1 = Set::IntSpan->new(sprintf("%d-%d",sort {$a <=> $b} ($coords[0]*1000,$coords[1]*1000)));
+    my $s2 = Set::IntSpan->new(sprintf("%d-%d",sort {$a <=> $b} ($coords[2]*1000,$coords[3]*1000)));
+    my $int = $s1->intersect($s2)->cardinality;
+    my $d   = span_distance(@coords);
+    if ($int) {
+      $int = ($int-1)/1000 if $int;
+      if ($int && (-$d - $int) > 0.002) {
+				die();
+      }
+    } else {
+      $int = min ( abs($coords[0] - $coords[2]),
+									 abs($coords[0] - $coords[3]),
+									 abs($coords[1] - $coords[2]),
+									 abs($coords[1] - $coords[3]) );
+      if ($d - $int > 0.002) {
+				die();
+      }
+    }
+    printinfo(@coords,$d,$int);
+  }
 }
 
 sub make_set {
 	my ($x,$y,%args) = @_;
-	if(! defined $x) {
+	if (! defined $x) {
 		return Set::IntSpan->new();
-	} elsif(! defined $y || $x == $y) {
+	} elsif (! defined $y || $x == $y) {
 		return Set::IntSpan->new(round $x);
 	} elsif ($x > $y) {
-		if($args{norev} || defined_and_zero($args{rev})) {
+		if ($args{norev} || defined_and_zero($args{rev})) {
 			fatal_error("data","malformed_span",$x,$y);
 		}
 		return Set::IntSpan->new( (round $y) . "-" . (round $x) );
@@ -606,6 +678,17 @@ sub false_or_no {
 	return if $x eq 0 || $x =~ /^no?$/i;
 }
 
+sub get_hash_leaf {
+	my ($hash,@path) = @_;
+	return $hash if ! @path;
+	my $key = shift @path;
+	if(! exists $hash->{$key}) {
+		fatal_error("system","hash_leaf_undef",$key);
+	} else {
+		return get_hash_leaf($hash->{$key},@path);
+	}
+}
+
 # -------------------------------------------------------------------
 # parse into a hash option string like
 #   var1=value1,var2=value2,...
@@ -621,4 +704,9 @@ sub parse_options {
   return $options;
 }
 
+sub log10 {
+	my $x = shift;
+	return $x > 0 ? log($x)/log(10) : undef;
+}
+
 1;
